@@ -87,15 +87,7 @@ static vector< string > receiveSocketMessage(int socket)
 	{
 		if (errno != ECONNRESET /* handled under */)
 		{
-			if (errno == EWOULDBLOCK || errno == EAGAIN)
-			{
-				// time is out
-				fatal("unable to receive socket message length: the connection died");
-			}
-			else
-			{
-				fatal("unable to receive socket message length: EEE");
-			}
+			fatal("unable to receive socket message length: EEE");
 		}
 	}
 	else if (readResult != sizeof(len) && readResult != 0)
@@ -180,6 +172,7 @@ class ManagerImpl
 	void proceedDownload(Pipe& workerPipe, const vector< string >& params, bool debugging);
 	void killPerformerBecauseOfWrongSize(Pipe& workerPipe, const string& uri,
 			const string& actionName, const string& errorString);
+	void terminateDownloadProcesses();
 	void startNewDownload(const string& uri, const string& targetPath, int waiterSocket, bool debugging);
 	int pollAllSockets(const vector< int >& persistentSockets, set< int >& clientSockets,
 			bool exitFlag, bool debugging);
@@ -265,11 +258,7 @@ ManagerImpl::ManagerImpl(const shared_ptr< const Config >& config_, const shared
 				debug("aborting download worker process because of exception");
 			}
 
-			// kill all forked processes
-			if (kill(0, SIGTERM) == -1)
-			{
-				warn("unable to send SIGTERM signal to download worker process group: EEE");
-			}
+			terminateDownloadProcesses();
 
 			_exit(EXIT_FAILURE);
 		}
@@ -314,6 +303,21 @@ ManagerImpl::~ManagerImpl()
 	}
 }
 
+void ManagerImpl::terminateDownloadProcesses()
+{
+	FORIT(activeDownloadIt, activeDownloads)
+	{
+		auto pid = activeDownloadIt->second.performerPid;
+		if (kill(pid, SIGTERM) == -1)
+		{
+			if (errno != ESRCH)
+			{
+				warn("download manager: unable to terminate a performer process (pid '%d')", pid);
+			}
+		}
+	}
+}
+
 // each worker has own process, so own ping pipe too
 Pipe pingPipe("worker's ping");
 volatile sig_atomic_t pingIsProcessed = true;
@@ -769,10 +773,6 @@ void ManagerImpl::worker()
 		debug("download worker process started");
 	}
 
-	if (setpgid(0, 0) == -1)
-	{
-		warn("unable to establish new download worker process group: setpgid failed: EEE");
-	}
 	parentPipe->useAsReader();
 	Pipe workerPipe("worker's own");
 
@@ -788,9 +788,6 @@ void ManagerImpl::worker()
 
 	// while caller may set exit flag, we should continue processing as long as
 	// something is pending in internal queue
-	pollfd workerReaderPollFd;
-	workerReaderPollFd.fd = workerPipe.getReaderFd();
-	workerReaderPollFd.events = POLLIN;
 	int sock;
 	while ((sock = pollAllSockets(persistentSockets, clientSockets, exitFlag, debugging)))
 	{
@@ -817,11 +814,7 @@ void ManagerImpl::worker()
 			{
 				// oh-ho, parent process exited before closing parentPipe socket...
 				// no reason to live anymore
-				// kill all forked processes including itself
-				if (kill(0, SIGTERM) == -1)
-				{
-					warn("unable to send SIGTERM signal to download worker process group: EEE");
-				}
+				terminateDownloadProcesses();
 				_exit(EXIT_FAILURE);
 			}
 			bool isPerformerSocket = false;
@@ -1021,6 +1014,31 @@ map< string, InnerDownloadElement > ManagerImpl::convertEntitiesToDownloads(
 	return result;
 }
 
+static void checkSocketForTimeout(int sock)
+{
+	pollfd pollFd;
+	pollFd.fd = sock;
+	pollFd.events = POLLIN;
+	do_poll:
+	auto pollResult = poll(&pollFd, 1, 2000); // 2 seconds
+	if (pollResult == -1)
+	{
+		if (errno == EINTR)
+		{
+			goto do_poll;
+		}
+		else
+		{
+			fatal("download client: polling client socket failed: EEE");
+		}
+	}
+	else if (!pollResult)
+	{
+		// time is out
+		fatal("download client: download server socket is timed out");
+	}
+}
+
 string ManagerImpl::download(const vector< Manager::DownloadEntity >& entities)
 {
 	if (config->getBool("cupt::worker::simulate"))
@@ -1051,15 +1069,6 @@ string ManagerImpl::download(const vector< Manager::DownloadEntity >& entities)
 	{
 		fatal("unable to connect to server socket: EEE");
 	}
-	{ // setting timeout
-		struct timeval tv;
-		tv.tv_sec = 2;
-		tv.tv_usec = 0;
-		if (setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) == -1)
-		{
-			warn("unable to set download client socket timeout: EEE");
-		}
-	}
 
 	auto scheduleDownload = [&sock, &downloads, &waitedUriToTargetPath](const string& targetPath)
 	{
@@ -1097,6 +1106,8 @@ string ManagerImpl::download(const vector< Manager::DownloadEntity >& entities)
 	string result; // no error by default
 	while (!waitedUriToTargetPath.empty())
 	{
+		checkSocketForTimeout(sock);
+
 		auto params = receiveSocketMessage(sock);
 		if (params.size() == 1 && params[0] == "ping")
 		{
@@ -1206,44 +1217,6 @@ Manager::~Manager()
 	delete __impl;
 }
 
-/*
-=head2 download
-
-method, adds group of download queries to queue. Blocks execution of program until
-all downloads are done.
-
-This method is re-entrant.
-
-Parameters:
-
-Returns:
-
-I<result> - '0' on success, otherwise the string that contains the fail reason,
-
-Example:
-
-  my $download_manager = new Cupt::Download::Manager;
-  $download_manager->download(
-    { 'uri-entries' => [ { 'uri' => 'http://www.en.debian.org' } ], 'filename' => '/tmp/en.html' },
-    { 'uri-entries' => [ { 'uri' => 'http://www.ru.debian.org' } ], 'filename' => '/tmp/ru.html', 'post-action' => \&checker },
-    { 'uri-entries' => [ { 'uri' => 'http://www.ua.debian.org' } ], 'filename' => '/tmp/ua.html', 'size' => 10254 }
-    { 'uri-entries' => [
-        {
-          'uri' => 'http://ftp.de.debian.org/debian/pool/main/n/nlkt/nlkt_0.3.2.1-2_amd64.deb',
-          'long-alias' => 'ftp.de.debian.org/debian sid/main nlkt 0.3.2.1-2',
-          'short-alias' => 'nlkt',
-        },
-        {
-          'uri' => 'http://ftp.ua.debian.org/debian/pool/main/n/nlkt/nlkt_0.3.2.1-2_amd64.deb',
-          'long-alias' => 'ftp.ua.debian.org/debian sid/main nlkt 0.3.2.1-2',
-          'short-alias' => 'nlkt',
-        },
-      ], 'filename' => '/var/cache/apt/archives/nlkt_0.3.2.1-2_amd64.deb' }
-  );
-
-=cut
-*/
-
 string Manager::download(const vector< DownloadEntity >& entities)
 {
 	return __impl->download(entities);
